@@ -85,6 +85,10 @@ class ControlReqDriver(BaseControlDriver):
             pass
         elif self.req.command == 'CANCEL':
             pass
+        elif self.req.command == 'DUMP':
+            self.logger.warning(
+                'DUMP command cannot be processed by ControlReqDriver. Use DumpDriver'
+            )
         elif self.req.command == 'RELOAD':
             if self.req.targets and self.runtime.__class__.__name__ == 'ZEDRuntime':
                 patterns = self.req.targets
