@@ -4,11 +4,25 @@ __license__ = "Apache-2.0"
 from typing import Any
 
 from .. import BaseExecutor
-from ..compound import CompoundExecutor
 
 if False:
     # fix type-hint complain for sphinx and flake
+    from typing import TypeVar
     import numpy as np
+    import scipy
+    import tensorflow as tf
+    import torch
+
+    EncodingType = TypeVar(
+        'EncodingType',
+        np.ndarray,
+        scipy.sparse.csr_matrix,
+        scipy.sparse.coo_matrix,
+        scipy.sparse.bsr_matrix,
+        scipy.sparse.csc_matrix,
+        torch.sparse_coo_tensor,
+        tf.SparseTensor,
+    )
 
 
 class BaseEncoder(BaseExecutor):
@@ -17,10 +31,10 @@ class BaseEncoder(BaseExecutor):
     The key function is :func:`encode`.
 
     .. seealso::
-        :mod:`jina.drivers.handlers.encode`
+        :mod:`jina.drivers.encode`
     """
 
-    def encode(self, data: Any, *args, **kwargs) -> Any:
+    def encode(self, data: Any, *args, **kwargs) -> 'EncodingType':
         """Encode the data, needs to be implemented in subclass.
         :param data: the data to be encoded
         :param args: additional positional arguments
@@ -33,7 +47,7 @@ class BaseEncoder(BaseExecutor):
 class BaseNumericEncoder(BaseEncoder):
     """BaseNumericEncoder encodes data from a ndarray, potentially B x ([T] x D) into a ndarray of B x D"""
 
-    def encode(self, data: 'np.ndarray', *args, **kwargs) -> 'np.ndarray':
+    def encode(self, data: 'np.ndarray', *args, **kwargs) -> 'EncodingType':
         """
         :param data: a `B x ([T] x D)` numpy ``ndarray``, `B` is the size of the batch
         :param args: additional positional arguments
@@ -44,16 +58,19 @@ class BaseNumericEncoder(BaseEncoder):
 
 class BaseImageEncoder(BaseNumericEncoder):
     """BaseImageEncoder encodes data from a ndarray, potentially B x (Height x Width) into a ndarray of B x D"""
+
     pass
 
 
 class BaseVideoEncoder(BaseNumericEncoder):
     """BaseVideoEncoder encodes data from a ndarray, potentially B x (Time x Height x Width) into a ndarray of B x D"""
+
     pass
 
 
 class BaseAudioEncoder(BaseNumericEncoder):
     """BaseAudioEncoder encodes data from a ndarray, potentially B x (Time x D) into a ndarray of B x D"""
+
     pass
 
 
@@ -62,7 +79,7 @@ class BaseTextEncoder(BaseEncoder):
     BaseTextEncoder encodes data from an array of string type (data.dtype.kind == 'U') of size B into a ndarray of B x D.
     """
 
-    def encode(self, data: 'np.ndarray', *args, **kwargs) -> 'np.ndarray':
+    def encode(self, data: 'np.ndarray', *args, **kwargs) -> 'EncodingType':
         """
 
         :param data: an 1d array of string type (data.dtype.kind == 'U') in size B
