@@ -3,13 +3,13 @@ __license__ = "Apache-2.0"
 
 import os
 import pickle
-from typing import List
+from typing import Sequence
 
 import numpy as np
 
 from jina.executors.devices import TorchDevice
 from jina.excepts import PretrainedModelFileDoesNotExist
-from jina.executors.decorators import batching_multi_input, as_ndarray
+from jina.executors.decorators import batching, as_ndarray
 from jina.executors.encoders.multimodal import BaseMultiModalEncoder
 
 
@@ -18,7 +18,7 @@ class TirgMultiModalEncoder(TorchDevice, BaseMultiModalEncoder):
     def __init__(self, model_path: str = 'checkpoint.pth',
                  texts_path: str = 'texts.pkl',
                  channel_axis: int = -1,
-                 positional_modality: List[str] = ['image', 'text'],
+                 positional_modality: Sequence[str] = ['image', 'text'],
                  *args, **kwargs):
         """
         :param model_path: the path where the model is stored.
@@ -64,7 +64,7 @@ class TirgMultiModalEncoder(TorchDevice, BaseMultiModalEncoder):
 
         return self.model.compose_img_text_features(img_features, text_features)
 
-    @batching_multi_input(num_data=2)
+    @batching(slice_nargs=2)
     @as_ndarray
     def encode(self, *data: 'np.ndarray', **kwargs) -> 'np.ndarray':
         feature = self._get_features(data).detach()
