@@ -1,8 +1,9 @@
 __copyright__ = "Copyright (c) 2021 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
-import os
-from typing import Optional
+import time
+
+from typing import Optional, Dict
 
 import numpy as np
 
@@ -10,6 +11,11 @@ from jina.executors.decorators import batching, as_ndarray
 from jina.executors.devices import TorchDevice
 from jina.executors.encoders import BaseEncoder
 
+if False:
+    import torch
+
+HTTP_SERVICE_UNAVAILABLE = 503
+
 
 class TransformerTorchEncoder(TorchDevice, BaseEncoder):
     """
@@ -28,7 +34,7 @@ class TransformerTorchEncoder(TorchDevice, BaseEncoder):
     :param layer_index: index of the transformer layer that is used to create
         encodings. Layer 0 corresponds to the embeddings layer
     :param max_length: the max length to truncate the tokenized sequences to.
-    :acceleration: The method to accelerate encoding. The available options are:
+    :param acceleration: The method to accelerate encoding. The available options are:
         - ``'amp'``, which uses `automatic mixed precision
             `<https://pytorch.org/docs/stable/amp.html>`_ autocasting.
             This option is only available on GPUs that support it
@@ -37,6 +43,8 @@ class TransformerTorchEncoder(TorchDevice, BaseEncoder):
             See `this tutorial
             <https://pytorch.org/tutorials/intermediate/dynamic_quantization_bert_tutorial.html>`_
             for more information. This option is currently not supported on GPUs.
+    :param embedding_fn_name: name of the function to be called from the `model` to do the embedding. `__call__` by default.
+            Other possible values would `embed_questions` for `RetriBert` based models
     :param args:  Additional positional arguments
     :param kwargs: Additional keyword arguments
 
@@ -47,23 +55,31 @@ class TransformerTorchEncoder(TorchDevice, BaseEncoder):
     """
 
     def __init__(
-            self,
-            pretrained_model_name_or_path: str = 'sentence-transformers/distilbert-base-nli-stsb-mean-tokens',
-            base_tokenizer_model: Optional[str] = None,
-            pooling_strategy: str = 'mean',
-            layer_index: int = -1,
-            max_length: Optional[int] = None,
-            acceleration: Optional[str] = None,
-            *args,
-            **kwargs,
+        self,
+        pretrained_model_name_or_path: str = 'sentence-transformers/distilbert-base-nli-stsb-mean-tokens',
+        base_tokenizer_model: Optional[str] = None,
+        pooling_strategy: str = 'mean',
+        layer_index: int = -1,
+        max_length: Optional[int] = None,
+        acceleration: Optional[str] = None,
+        embedding_fn_name: str = '__call__',
+        api_token: Optional[str] = None,
+        max_retries: int = 20,
+        *args,
+        **kwargs,
     ):
         super().__init__(*args, **kwargs)
         self.pretrained_model_name_or_path = pretrained_model_name_or_path
-        self.base_tokenizer_model = base_tokenizer_model or pretrained_model_name_or_path
+        self.base_tokenizer_model = (
+            base_tokenizer_model or pretrained_model_name_or_path
+        )
         self.pooling_strategy = pooling_strategy
         self.layer_index = layer_index
         self.max_length = max_length
         self.acceleration = acceleration
+        self.embedding_fn_name = embedding_fn_name
+        self.api_token = api_token
+        self.max_retries = max_retries
 
         if self.pooling_strategy == 'auto':
             self.pooling_strategy = 'cls'
@@ -86,21 +102,32 @@ class TransformerTorchEncoder(TorchDevice, BaseEncoder):
             )
             raise NotImplementedError
 
+        if self.api_token is not None and self.layer_index != -1:
+            self.logger.error(
+                f'layer_index {self.layer_index} not available via the huggingface API.'
+                ' Please set the layer_index to -1 or disable huggingface API usage.'
+            )
+            raise ValueError
+
     def post_init(self):
         """Load the transformer model and encoder"""
         import torch
         from transformers import AutoModel, AutoTokenizer
 
         self.tokenizer = AutoTokenizer.from_pretrained(self.base_tokenizer_model)
-        self.model = AutoModel.from_pretrained(
-            self.pretrained_model_name_or_path, output_hidden_states=True
-        )
-        self.to_device(self.model)
 
-        if self.acceleration == 'quant' and not self.on_gpu:
-            self.model = torch.quantization.quantize_dynamic(
-                self.model, {torch.nn.Linear}, dtype=torch.qint8
+        if self.api_token is None:
+            self.model = AutoModel.from_pretrained(
+                self.pretrained_model_name_or_path, output_hidden_states=True
             )
+            self.to_device(self.model)
+
+            if self.acceleration == 'quant' and not self.on_gpu:
+                self.model = torch.quantization.quantize_dynamic(
+                    self.model, {torch.nn.Linear}, dtype=torch.qint8
+                )
+        else:
+            self._api_call('Scotty, please warmup.')
 
     def amp_accelerate(self):
         """Check acceleration method """
@@ -112,6 +139,64 @@ class TransformerTorchEncoder(TorchDevice, BaseEncoder):
         else:
             return nullcontext()
 
+    def _api_call(self, query):
+        import requests
+
+        retries = 0
+        headers = {"Authorization": f"Bearer {self.api_token}"}
+        api_url = f"https://api-inference.huggingface.co/pipeline/feature-extraction/{self.pretrained_model_name_or_path}"
+
+        while retries < self.max_retries:
+            retries += 1
+            r = requests.post(api_url, json=query, headers=headers)
+            if r.status_code == HTTP_SERVICE_UNAVAILABLE:
+                self.logger.info('Service is currently unavailable. Model is loading.')
+                time.sleep(3)
+            else:
+                break
+        return r.json()
+
+    def _compute_embedding(self, hidden_states: 'torch.Tensor', input_tokens: Dict):
+        import torch
+
+        n_layers = len(hidden_states)
+        if self.layer_index not in list(range(-n_layers, n_layers)):
+            self.logger.error(
+                f'Invalid value {self.layer_index} for `layer_index`,'
+                f' for the model {self.pretrained_model_name_or_path}'
+                f' valid values are integers from {-n_layers} to {n_layers - 1}.'
+            )
+            raise ValueError
+
+        if self.pooling_strategy == 'cls' and not self.tokenizer.cls_token:
+            self.logger.error(
+                f'You have set pooling_strategy to "cls", but the tokenizer'
+                f' for the model {self.pretrained_model_name_or_path}'
+                f' does not have a cls token set.'
+            )
+            raise ValueError
+
+        fill_vals = {'cls': 0.0, 'mean': 0.0, 'max': -np.inf, 'min': np.inf}
+        fill_val = torch.tensor(fill_vals[self.pooling_strategy], device=self.device)
+
+        layer = hidden_states[self.layer_index]
+        attn_mask = input_tokens['attention_mask'].unsqueeze(-1).expand_as(layer)
+        layer = torch.where(attn_mask.bool(), layer, fill_val)
+
+        if self.pooling_strategy == 'cls':
+            CLS = self.tokenizer.cls_token_id
+            ind = torch.nonzero(input_tokens['input_ids'] == CLS)[:, 1]
+            ind = ind.unsqueeze(-1).unsqueeze(-1).expand(-1, -1, layer.shape[2])
+            embeddings = torch.gather(layer, 1, ind).squeeze(dim=1)
+        elif self.pooling_strategy == 'mean':
+            embeddings = layer.sum(dim=1) / attn_mask.sum(dim=1)
+        elif self.pooling_strategy == 'max':
+            embeddings = layer.max(dim=1).values
+        elif self.pooling_strategy == 'min':
+            embeddings = layer.min(dim=1).values
+
+        return embeddings.cpu().numpy()
+
     @batching
     @as_ndarray
     def encode(self, data: 'np.ndarray', *args, **kwargs) -> 'np.ndarray':
@@ -139,43 +224,17 @@ class TransformerTorchEncoder(TorchDevice, BaseEncoder):
             )
             input_tokens = {k: v.to(self.device) for k, v in input_tokens.items()}
 
-            with self.amp_accelerate():
-                outputs = self.model(**input_tokens)
+            if self.api_token is not None:
+                outputs = self._api_call(list(data))
 
-            n_layers = len(outputs.hidden_states)
-            if self.layer_index not in list(range(-n_layers, n_layers)):
-                self.logger.error(
-                    f'Invalid value {self.layer_index} for `layer_index`,'
-                    f' for the model {self.pretrained_model_name_or_path}'
-                    f' valid values are integers from {-n_layers} to {n_layers - 1}.'
-                )
-                raise ValueError
+                hidden_states = torch.tensor([outputs])
+            else:
+                with self.amp_accelerate():
+                    outputs = getattr(self.model, self.embedding_fn_name)(
+                        **input_tokens
+                    )
+                    if isinstance(outputs, torch.Tensor):
+                        return outputs.cpu().numpy()
+                    hidden_states = outputs.hidden_states
 
-            if self.pooling_strategy == 'cls' and not self.tokenizer.cls_token:
-                self.logger.error(
-                    f'You have set pooling_strategy to "cls", but the tokenizer'
-                    f' for the model {self.pretrained_model_name_or_path}'
-                    f' does not have a cls token set.'
-                )
-                raise ValueError
-
-            fill_vals = {'cls': 0.0, 'mean': 0.0, 'max': -np.inf, 'min': np.inf}
-            fill_val = torch.tensor(fill_vals[self.pooling_strategy], device=self.device)
-
-            layer = outputs.hidden_states[self.layer_index]
-            attn_mask = input_tokens['attention_mask'].unsqueeze(-1).expand_as(layer)
-            layer = torch.where(attn_mask.bool(), layer, fill_val)
-
-            if self.pooling_strategy == 'cls':
-                CLS = self.tokenizer.cls_token_id
-                ind = torch.nonzero(input_tokens['input_ids'] == CLS)[:, 1]
-                ind = ind.unsqueeze(-1).unsqueeze(-1).expand(-1, -1, layer.shape[2])
-                embeddings = torch.gather(layer, 1, ind).squeeze(dim=1)
-            elif self.pooling_strategy == 'mean':
-                embeddings = layer.sum(dim=1) / attn_mask.sum(dim=1)
-            elif self.pooling_strategy == 'max':
-                embeddings = layer.max(dim=1).values
-            elif self.pooling_strategy == 'min':
-                embeddings = layer.min(dim=1).values
-
-            return embeddings.cpu().numpy()
+            return self._compute_embedding(hidden_states, input_tokens)
