@@ -1,15 +1,12 @@
-from enum import Enum
-from datetime import datetime
-from types import SimpleNamespace
 from collections import defaultdict
+from datetime import datetime
+from enum import Enum
+from types import SimpleNamespace
 from typing import Callable, Dict, Any, Optional, List, Union
 
-from pydantic import Field, BaseModel, BaseConfig, create_model, root_validator
 from google.protobuf.descriptor import Descriptor, FieldDescriptor
 from google.protobuf.pyext.cpp_message import GeneratedProtocolMessageType
-
 from jina.enums import DataInputType
-from jina.types.document import Document
 from jina.parsers import set_client_cli_parser
 from jina.proto.jina_pb2 import (
     DenseNdArrayProto,
@@ -24,6 +21,8 @@ from jina.proto.jina_pb2 import (
     RequestProto,
     QueryLangProto,
 )
+from jina.types.document import Document
+from pydantic import Field, BaseModel, BaseConfig, create_model, root_validator
 
 DEFAULT_REQUEST_SIZE = set_client_cli_parser().parse_args([]).request_size
 PROTO_TO_PYDANTIC_MODELS = SimpleNamespace()
@@ -104,7 +103,6 @@ def _get_tags_updater() -> Callable:
     :return: classmethod for updating tags in DocumentProto Pydantic model
     """
 
-    @root_validator(pre=True)
     def tags_updater(cls, values):
         extra_fields = {k: values[k] for k in set(values).difference(cls.__fields__)}
         if extra_fields:
@@ -239,6 +237,14 @@ class JinaStatusModel(BaseModel):
     used_memory: str
 
 
+class JinaReloadRequestModel(BaseModel):
+    """
+    Jina control request model.
+    """
+
+    targets: Union[str, List[str]]
+
+
 class JinaRequestModel(BaseModel):
     """
     Jina request model.
@@ -258,6 +264,40 @@ class JinaRequestModel(BaseModel):
     queryset: Optional[List[PROTO_TO_PYDANTIC_MODELS.QueryLangProto]] = None
     data_type: DataInputType = DataInputType.AUTO
 
+    @root_validator(pre=True, allow_reuse=True)
+    def add_default_kwargs(cls, kwargs: dict):
+        """
+        Replicates jina.clients.base.BaseClient.add_default_kwargs for Pydantic
+
+        :param kwargs: arguments passed to the Pydantic model
+        :type kwargs: dict
+        :return: kwargs
+        """
+        if ('top_k' in kwargs) and (kwargs['top_k'] is not None):
+            # associate all VectorSearchDriver and SliceQL driver to use top_k
+            topk_ql = [
+                PROTO_TO_PYDANTIC_MODELS.QueryLangProto(
+                    **{
+                        'name': 'SliceQL',
+                        'priority': 1,
+                        'parameters': {'end': kwargs['top_k']},
+                    }
+                ),
+                PROTO_TO_PYDANTIC_MODELS.QueryLangProto(
+                    **{
+                        'name': 'VectorSearchDriver',
+                        'priority': 1,
+                        'parameters': {'top_k': kwargs['top_k']},
+                    }
+                ),
+            ]
+            if 'queryset' not in kwargs:
+                kwargs['queryset'] = topk_ql
+            else:
+                kwargs['queryset'].extend(topk_ql)
+
+        return kwargs
+
 
 class JinaIndexRequestModel(JinaRequestModel):
     """Index request model."""
