@@ -5,7 +5,8 @@ _schema_flow_with = _cli_to_schema(
     api_to_dict(),
     'flow',
     allow_addition=False,
-    description='The config of Flow, unrecognized config arguments will be applied to all Pods')['Jina::Flow']
+    description='The config of Flow, unrecognized config arguments will be applied to all Pods',
+)['Jina::Flow']
 
 schema_flow = {
     'Jina::Flow': {
@@ -13,11 +14,11 @@ schema_flow = {
             'with': _schema_flow_with,
             'jtype': {
                 'description': 'The type of Jina object (Flow, Executor, Driver).\n'
-                               'A Flow is made up of several sub-tasks, and it manages the states and context of these sub-tasks.\n'
-                               'The input and output data of Flows are Documents.',
+                'A Flow is made up of several sub-tasks, and it manages the states and context of these sub-tasks.\n'
+                'The input and output data of Flows are Documents.',
                 'type': 'string',
                 'default': 'Flow',
-                'enum': ['Flow', 'AsyncFlow']
+                'enum': ['Flow', 'AsyncFlow'],
             },
             'version': {
                 'description': 'The YAML version of this Flow.',
@@ -26,15 +27,14 @@ schema_flow = {
             },
             'pods': {
                 'description': 'Define the steps in the Flow.\n'
-                               'A Pod is a container and interface for one or multiple Peas that have the same properties.',
+                'A Pod is a container and interface for one or multiple Peas that have the same properties.',
                 'type': 'array',
-                'items': {
-                    '$ref': '#/definitions/Jina::Pod'
-                },
-                'minItems': 1
-            }
+                'items': {'$ref': '#/definitions/Jina::Pod'},
+                'minItems': 1,
+            },
         },
         'type': 'object',
         'additionalProperties': False,
         'required': ['jtype', 'version', 'pods'],
-    }}
+    }
+}
