@@ -7,7 +7,7 @@ import urllib.parse
 import urllib.request
 import warnings
 from hashlib import blake2b
-from typing import Union, Dict, Optional, TypeVar, Any, Tuple, List
+from typing import Union, Dict, Optional, TypeVar, Any, Tuple, List, Type
 
 import numpy as np
 from google.protobuf import json_format
@@ -16,7 +16,7 @@ from google.protobuf.field_mask_pb2 import FieldMask
 from .traversable import Traversable
 from .converters import png_to_buffer, to_datauri, guess_mime, to_image_blob
 from ..mixin import ProtoTypeMixin
-from ..ndarray.generic import NdArray
+from ..ndarray.generic import NdArray, BaseSparseNdArray
 from ..querylang.queryset.dunderkey import dunder_get
 from ..score import NamedScore
 from ..sets.chunk import ChunkSet
@@ -27,6 +27,38 @@ from ...importer import ImportExtensions
 from ...logging import default_logger
 from ...proto import jina_pb2
 
+if False:
+    from scipy.sparse import coo_matrix
+
+    # fix type-hint complain for sphinx and flake
+    from typing import TypeVar
+    import numpy as np
+    import scipy
+    import tensorflow as tf
+    import torch
+
+    EmbeddingType = TypeVar(
+        'EncodingType',
+        np.ndarray,
+        scipy.sparse.csr_matrix,
+        scipy.sparse.coo_matrix,
+        scipy.sparse.bsr_matrix,
+        scipy.sparse.csc_matrix,
+        torch.sparse_coo_tensor,
+        tf.SparseTensor,
+    )
+
+    SparseEmbeddingType = TypeVar(
+        'SparseEmbeddingType',
+        np.ndarray,
+        scipy.sparse.csr_matrix,
+        scipy.sparse.coo_matrix,
+        scipy.sparse.bsr_matrix,
+        scipy.sparse.csc_matrix,
+        torch.sparse_coo_tensor,
+        tf.SparseTensor,
+    )
+
 __all__ = ['Document', 'DocumentContentType', 'DocumentSourceType']
 DIGEST_SIZE = 8
 
@@ -205,7 +237,7 @@ class Document(ProtoTypeMixin, Traversable):
         self._mermaid_id = random_identity()  #: for mermaid visualize id
 
     @property
-    def length(self) -> int:
+    def siblings(self) -> int:
         """
         The number of siblings of the :class:``Document``
 
@@ -214,13 +246,11 @@ class Document(ProtoTypeMixin, Traversable):
         :setter: number of siblings
         :type: int
         """
-        # TODO(Han): rename this to siblings as this shadows the built-in `length`
+        return self._pb_body.siblings
 
-        return self._pb_body.length
-
-    @length.setter
-    def length(self, value: int):
-        self._pb_body.length = value
+    @siblings.setter
+    def siblings(self, value: int):
+        self._pb_body.siblings = value
 
     @property
     def weight(self) -> float:
@@ -469,13 +499,31 @@ class Document(ProtoTypeMixin, Traversable):
         self._update_ndarray('blob', value)
 
     @property
-    def embedding(self) -> 'np.ndarray':
+    def embedding(self) -> 'EmbeddingType':
         """Return ``embedding`` of the content of a Document.
 
         :return: the embedding from the proto
         """
         return NdArray(self._pb_body.embedding).value
 
+    def get_sparse_embedding(
+        self, sparse_ndarray_cls_type: Type[BaseSparseNdArray], **kwargs
+    ) -> 'SparseEmbeddingType':
+        """Return ``embedding`` of the content of a Document as an sparse array.
+
+        :param sparse_ndarray_cls_type: Sparse class type, such as `SparseNdArray`.
+        :param kwargs: Additional key value argument, for `scipy` backend, we need to set
+            the keyword `sp_format` as one of the scipy supported sparse format, such as `coo`
+            or `csr`.
+        :return: the embedding from the proto as an sparse array
+        """
+        return NdArray(
+            self._pb_body.embedding,
+            sparse_cls=sparse_ndarray_cls_type,
+            is_sparse=True,
+            **kwargs,
+        ).value
+
     @embedding.setter
     def embedding(self, value: Union['np.ndarray', 'jina_pb2.NdArrayProto', 'NdArray']):
         """Set the ``embedding`` of the content of a Document.
@@ -484,6 +532,73 @@ class Document(ProtoTypeMixin, Traversable):
         """
         self._update_ndarray('embedding', value)
 
+    def _update_sparse_ndarray(self, k, v, sparse_cls):
+        NdArray(
+            is_sparse=True,
+            sparse_cls=sparse_cls,
+            proto=getattr(self._pb_body, k),
+        ).value = v
+
+    def _check_installed_array_packages(self):
+        from ... import JINA_GLOBAL
+
+        if JINA_GLOBAL.scipy_installed is None:
+            JINA_GLOBAL.scipy_installed = False
+            with ImportExtensions(required=False, pkg_name='scipy'):
+                import scipy
+
+                JINA_GLOBAL.scipy_installed = True
+
+        if JINA_GLOBAL.tensorflow_installed is None:
+            JINA_GLOBAL.tensorflow_installed = False
+            with ImportExtensions(required=False, pkg_name='tensorflow'):
+                import tensorflow
+
+                JINA_GLOBAL.tensorflow_installed = True
+
+        if JINA_GLOBAL.torch_installed is None:
+            JINA_GLOBAL.torch_installed = False
+            with ImportExtensions(required=False, pkg_name='torch'):
+                import torch
+
+                JINA_GLOBAL.torch_installed = True
+
+    def _update_if_sparse(self, k, v):
+
+        from ... import JINA_GLOBAL
+
+        v_valid_sparse_type = False
+        self._check_installed_array_packages()
+
+        if JINA_GLOBAL.scipy_installed:
+            import scipy
+
+            if scipy.sparse.issparse(v):
+                from ..ndarray.sparse.scipy import SparseNdArray
+
+                self._update_sparse_ndarray(k=k, v=v, sparse_cls=SparseNdArray)
+                v_valid_sparse_type = True
+
+        if JINA_GLOBAL.tensorflow_installed:
+            import tensorflow
+
+            if isinstance(v, tensorflow.SparseTensor):
+                from ..ndarray.sparse.tensorflow import SparseNdArray
+
+                self._update_sparse_ndarray(k=k, v=v, sparse_cls=SparseNdArray)
+                v_valid_sparse_type = True
+
+        if JINA_GLOBAL.torch_installed:
+            import torch
+
+            if isinstance(v, torch.Tensor) and v.is_sparse:
+                from ..ndarray.sparse.pytorch import SparseNdArray
+
+                self._update_sparse_ndarray(k=k, v=v, sparse_cls=SparseNdArray)
+                v_valid_sparse_type = True
+
+        return v_valid_sparse_type
+
     def _update_ndarray(self, k, v):
         if isinstance(v, jina_pb2.NdArrayProto):
             getattr(self._pb_body, k).CopyFrom(v)
@@ -492,8 +607,12 @@ class Document(ProtoTypeMixin, Traversable):
         elif isinstance(v, NdArray):
             NdArray(getattr(self._pb_body, k)).is_sparse = v.is_sparse
             NdArray(getattr(self._pb_body, k)).value = v.value
+
         else:
-            raise TypeError(f'{k} is in unsupported type {typename(v)}')
+            v_valid_sparse_type = self._update_if_sparse(k, v)
+
+            if not v_valid_sparse_type:
+                raise TypeError(f'{k} is in unsupported type {typename(v)}')
 
     @property
     def matches(self) -> 'MatchSet':
@@ -739,7 +858,7 @@ class Document(ProtoTypeMixin, Traversable):
             if r:
                 self._pb_body.mime_type = r
             else:
-                raise ValueError(f'{value} is not a valid MIME type')
+                self._pb_body.mime_type = value
 
     def __enter__(self):
         return self
