@@ -126,7 +126,9 @@ class Message:
 
         :return: boolean which states if data is requested
         """
-        return self.envelope.request_type != 'ControlRequest'
+        return (
+            self.envelope.request_type != 'ControlRequest' or self.request.propagate
+        ) and self.envelope.request_type != 'DumpRequest'
 
     def _add_envelope(
         self,
@@ -150,7 +152,7 @@ class Message:
         :param pod_name: the name of the current pod
         :param identity: the identity of the current pod
         :param check_version: turn on check_version
-        :param args:  Additional positional arguments
+        :param args: Additional positional arguments
         :param kwargs: Additional keyword arguments
         :param request_id: request id of the envelope
         :param request_type: request type of the envelope
